@@ -48,6 +48,8 @@ class AdminSite(object):
     # URL for the "View site" link at the top of each admin page.
     site_url = '/'
 
+    _empty_value_display = '-'
+
     login_form = None
     index_template = None
     app_index_template = None
@@ -101,11 +103,12 @@ class AdminSite(object):
                     options['__module__'] = __name__
                     admin_class = type("%sAdmin" % model.__name__, (admin_class,), options)
 
-                if admin_class is not ModelAdmin and settings.DEBUG:
-                    system_check_errors.extend(admin_class.check(model))
-
                 # Instantiate the admin class to save in the registry
-                self._registry[model] = admin_class(model, self)
+                admin_obj = admin_class(model, self)
+                if admin_class is not ModelAdmin and settings.DEBUG:
+                    system_check_errors.extend(admin_obj.check())
+
+                self._registry[model] = admin_obj
 
     def unregister(self, model_or_iterable):
         """
@@ -154,6 +157,14 @@ class AdminSite(object):
         """
         return six.iteritems(self._actions)
 
+    @property
+    def empty_value_display(self):
+        return self._empty_value_display
+
+    @empty_value_display.setter
+    def empty_value_display(self, empty_value_display):
+        self._empty_value_display = empty_value_display
+
     def has_permission(self, request):
         """
         Returns True if the given HttpRequest has permission to view
@@ -252,6 +263,7 @@ class AdminSite(object):
         def wrap(view, cacheable=False):
             def wrapper(*args, **kwargs):
                 return self.admin_view(view, cacheable)(*args, **kwargs)
+            wrapper.admin_site = self
             return update_wrapper(wrapper, view)
 
         # Admin-site-wide views.
@@ -270,7 +282,7 @@ class AdminSite(object):
         # Add in each model's views, and create a list of valid URLS for the
         # app_index
         valid_app_labels = []
-        for model, model_admin in six.iteritems(self._registry):
+        for model, model_admin in self._registry.items():
             urlpatterns += [
                 url(r'^%s/%s/' % (model._meta.app_label, model._meta.model_name), include(model_admin.urls)),
             ]
@@ -300,6 +312,7 @@ class AdminSite(object):
             'site_header': self.site_header,
             'site_url': self.site_url,
             'has_permission': self.has_permission(request),
+            'available_apps': self.get_app_list(request),
         }
 
     def password_change(self, request, extra_context=None):
@@ -310,13 +323,13 @@ class AdminSite(object):
         from django.contrib.auth.views import password_change
         url = reverse('admin:password_change_done', current_app=self.name)
         defaults = {
-            'current_app': self.name,
             'password_change_form': AdminPasswordChangeForm,
             'post_change_redirect': url,
             'extra_context': dict(self.each_context(request), **(extra_context or {})),
         }
         if self.password_change_template is not None:
             defaults['template_name'] = self.password_change_template
+        request.current_app = self.name
         return password_change(request, **defaults)
 
     def password_change_done(self, request, extra_context=None):
@@ -325,11 +338,11 @@ class AdminSite(object):
         """
         from django.contrib.auth.views import password_change_done
         defaults = {
-            'current_app': self.name,
             'extra_context': dict(self.each_context(request), **(extra_context or {})),
         }
         if self.password_change_done_template is not None:
             defaults['template_name'] = self.password_change_done_template
+        request.current_app = self.name
         return password_change_done(request, **defaults)
 
     def i18n_javascript(self, request):
@@ -354,11 +367,17 @@ class AdminSite(object):
         """
         from django.contrib.auth.views import logout
         defaults = {
-            'current_app': self.name,
-            'extra_context': dict(self.each_context(request), **(extra_context or {})),
+            'extra_context': dict(
+                self.each_context(request),
+                # Since the user isn't logged out at this point, the value of
+                # has_permission must be overridden.
+                has_permission=False,
+                **(extra_context or {})
+            ),
         }
         if self.logout_template is not None:
             defaults['template_name'] = self.logout_template
+        request.current_app = self.name
         return logout(request, **defaults)
 
     @never_cache
@@ -382,72 +401,107 @@ class AdminSite(object):
         )
         if (REDIRECT_FIELD_NAME not in request.GET and
                 REDIRECT_FIELD_NAME not in request.POST):
-            context[REDIRECT_FIELD_NAME] = request.get_full_path()
+            context[REDIRECT_FIELD_NAME] = reverse('admin:index', current_app=self.name)
         context.update(extra_context or {})
 
         defaults = {
             'extra_context': context,
-            'current_app': self.name,
             'authentication_form': self.login_form or AdminAuthenticationForm,
             'template_name': self.login_template or 'admin/login.html',
         }
+        request.current_app = self.name
         return login(request, **defaults)
 
+    def _build_app_dict(self, request, label=None):
+        """
+        Builds the app dictionary. Takes an optional label parameters to filter
+        models of a specific app.
+        """
+        app_dict = {}
+
+        if label:
+            models = {
+                m: m_a for m, m_a in self._registry.items()
+                if m._meta.app_label == label
+            }
+        else:
+            models = self._registry
+
+        for model, model_admin in models.items():
+            app_label = model._meta.app_label
+
+            has_module_perms = model_admin.has_module_permission(request)
+            if not has_module_perms:
+                if label:
+                    raise PermissionDenied
+                continue
+
+            perms = model_admin.get_model_perms(request)
+
+            # Check whether user has any perm for this module.
+            # If so, add the module to the model_list.
+            if True not in perms.values():
+                continue
+
+            info = (app_label, model._meta.model_name)
+            model_dict = {
+                'name': capfirst(model._meta.verbose_name_plural),
+                'object_name': model._meta.object_name,
+                'perms': perms,
+            }
+            if perms.get('change'):
+                try:
+                    model_dict['admin_url'] = reverse('admin:%s_%s_changelist' % info, current_app=self.name)
+                except NoReverseMatch:
+                    pass
+            if perms.get('add'):
+                try:
+                    model_dict['add_url'] = reverse('admin:%s_%s_add' % info, current_app=self.name)
+                except NoReverseMatch:
+                    pass
+
+            if app_label in app_dict:
+                app_dict[app_label]['models'].append(model_dict)
+            else:
+                app_dict[app_label] = {
+                    'name': apps.get_app_config(app_label).verbose_name,
+                    'app_label': app_label,
+                    'app_url': reverse(
+                        'admin:app_list',
+                        kwargs={'app_label': app_label},
+                        current_app=self.name,
+                    ),
+                    'has_module_perms': has_module_perms,
+                    'models': [model_dict],
+                }
+
+        if label:
+            return app_dict.get(label)
+        return app_dict
+
+    def get_app_list(self, request):
+        """
+        Returns a sorted list of all the installed apps that have been
+        registered in this site.
+        """
+        app_dict = self._build_app_dict(request)
+
+        # Sort the apps alphabetically.
+        app_list = sorted(app_dict.values(), key=lambda x: x['name'].lower())
+
+        # Sort the models alphabetically within each app.
+        for app in app_list:
+            app['models'].sort(key=lambda x: x['name'])
+
+        return app_list
+
     @never_cache
     def index(self, request, extra_context=None):
         """
         Displays the main admin index page, which lists all of the installed
         apps that have been registered in this site.
         """
-        app_dict = {}
-        for model, model_admin in self._registry.items():
-            app_label = model._meta.app_label
-            has_module_perms = model_admin.has_module_permission(request)
-
-            if has_module_perms:
-                perms = model_admin.get_model_perms(request)
-
-                # Check whether user has any perm for this module.
-                # If so, add the module to the model_list.
-                if True in perms.values():
-                    info = (app_label, model._meta.model_name)
-                    model_dict = {
-                        'name': capfirst(model._meta.verbose_name_plural),
-                        'object_name': model._meta.object_name,
-                        'perms': perms,
-                    }
-                    if perms.get('change', False):
-                        try:
-                            model_dict['admin_url'] = reverse('admin:%s_%s_changelist' % info, current_app=self.name)
-                        except NoReverseMatch:
-                            pass
-                    if perms.get('add', False):
-                        try:
-                            model_dict['add_url'] = reverse('admin:%s_%s_add' % info, current_app=self.name)
-                        except NoReverseMatch:
-                            pass
-                    if app_label in app_dict:
-                        app_dict[app_label]['models'].append(model_dict)
-                    else:
-                        app_dict[app_label] = {
-                            'name': apps.get_app_config(app_label).verbose_name,
-                            'app_label': app_label,
-                            'app_url': reverse(
-                                'admin:app_list',
-                                kwargs={'app_label': app_label},
-                                current_app=self.name,
-                            ),
-                            'has_module_perms': has_module_perms,
-                            'models': [model_dict],
-                        }
-
-        # Sort the apps alphabetically.
-        app_list = list(six.itervalues(app_dict))
-        app_list.sort(key=lambda x: x['name'].lower())
-
-        # Sort the models alphabetically within each app.
-        for app in app_list:
-            app['models'].sort(key=lambda x: x['name'])
+        app_list = self.get_app_list(request)
 
         context = dict(
             self.each_context(request),
@@ -462,52 +516,12 @@ class AdminSite(object):
                                 'admin/index.html', context)
 
     def app_index(self, request, app_label, extra_context=None):
-        app_name = apps.get_app_config(app_label).verbose_name
-        app_dict = {}
-        for model, model_admin in self._registry.items():
-            if app_label == model._meta.app_label:
-                has_module_perms = model_admin.has_module_permission(request)
-                if not has_module_perms:
-                    raise PermissionDenied
-
-                perms = model_admin.get_model_perms(request)
-
-                # Check whether user has any perm for this module.
-                # If so, add the module to the model_list.
-                if True in perms.values():
-                    info = (app_label, model._meta.model_name)
-                    model_dict = {
-                        'name': capfirst(model._meta.verbose_name_plural),
-                        'object_name': model._meta.object_name,
-                        'perms': perms,
-                    }
-                    if perms.get('change'):
-                        try:
-                            model_dict['admin_url'] = reverse('admin:%s_%s_changelist' % info, current_app=self.name)
-                        except NoReverseMatch:
-                            pass
-                    if perms.get('add'):
-                        try:
-                            model_dict['add_url'] = reverse('admin:%s_%s_add' % info, current_app=self.name)
-                        except NoReverseMatch:
-                            pass
-                    if app_dict:
-                        app_dict['models'].append(model_dict),
-                    else:
-                        # First time around, now that we know there's
-                        # something to display, add in the necessary meta
-                        # information.
-                        app_dict = {
-                            'name': app_name,
-                            'app_label': app_label,
-                            'app_url': '',
-                            'has_module_perms': has_module_perms,
-                            'models': [model_dict],
-                        }
+        app_dict = self._build_app_dict(request, app_label)
         if not app_dict:
             raise Http404('The requested admin page does not exist.')
         # Sort the models alphabetically within each app.
         app_dict['models'].sort(key=lambda x: x['name'])
+        app_name = apps.get_app_config(app_label).verbose_name
         context = dict(self.each_context(request),
             title=_('%(app)s administration') % {'app': app_name},
             app_list=[app_dict],
